package OnlyOrNo;
#
# $Id: OnlyOrNo.pm 249 2007-02-05 16:19:02Z gouzy $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

OnlyOrNo.pm - This object manages the switches --only and --no in the comand line

=cut


use strict;	
use English;
use Carp;
use vars qw(@ISA @EXPORT);
require Exporter;

@ISA = qw(Exporter);

sub BEGIN{
    our $VERSION = do {my @r = (q$Rev: 249 $ =~ /\d+/g); $r[0]};
}


=head2 New 

 Usage:     my $o_onlyno = New OnlyOrNo ($o_param);
 Procedure: Initialize an object from an already initialized ParamPArser
            The parameters are analyzed and '--only_xxx' parameters, as well as '--no_xxx' parameters, are looked for
            If none is found or is all are false, we switch to the complete mode.
            If at least one 'only_XXX' parameter is found, we switch to the 'only' mode
            If at least one 'no_XXX' parameter is found, we switch to the 'nono' mode
            If at least one 'only_xxx' AND one 'no_xxx' parameter are found, we die with an error message.
 Return:    the created object
 Args:      $o_param, the param parser object

=cut

sub New 
{
    my ($class, $o_param) = @_;
    my $self = bless {}, ref($class) || $class;

    confess("Please specify a ParamParser object")   if (!defined($o_param));

    my $cnt_only=0;
    my $cnt_no = 0;
    my $complete_mode=1;
    my $only_mode= 0;
    my @only_xxx = $o_param->GetKeys('only_');
    foreach my $p (@only_xxx) {
        if ($o_param->Get($p))
        {
            $only_mode=1;
            $complete_mode=0;
            $cnt_only++;
        };
    };

    my $nono_mode= 0;
    my @no_xxx = $o_param->GetKeys('no_');
    foreach my $p (@no_xxx) {
        if ($o_param->Get($p))
        {
            $nono_mode=1;
            $complete_mode=0;
            $cnt_no++;
        };
    };
    
    if ($nono_mode==1 and $only_mode==1) 
    {
        die "Please DO NOT specify --only_ switches WITH --no_ switches";
    };
    
    $self->{__complete_mode} = $complete_mode;
    $self->{__only_mode} = $only_mode;
    $self->{__nono_mode} = $nono_mode;
    $self->{__o_param}   = $o_param;
    $self->{__cnt_only}  = $cnt_only;
    $self->{__cnt_no}    = $cnt_no;
    
    return $self;
}

=head2 Get 

 Usage: if ($o_onlyno->Get('some_switch))... else ...
 Function: For one some_switch, return 0/1 in the following cases:
           -Return 0 if no_some_switch and only_some_switch are unknown of $o_param
           -Return 1 if we are in complete mode
           -Return 1 if we are in only mode and --only_some_switch is set
           -Return 0 if we are in only mode and --only_some_switch is NOT set
           -Return 0 if we are in nono_mode and --no_some_switch is set
           -Return 1 if we are in nono_mode and --no_some_switch is NOT set

 Args:     The switch to check for only_ form or no_ form

=cut

sub Get
{
    my $self   = shift;
    my $switch = shift;
    my $only = 'only_'.$switch;
    my $nono = 'no_'.$switch;
    my $o_param = $self->{__o_param};
    return 1 if ($self->{__complete_mode}==1);
    return 1 if ($self->{__only_mode}==1 and $o_param->Get($only));
    return 1 if ($self->{__nono_mode}==1 and !$o_param->Get($nono));
    return 0;
};

=head2 HowManyOnly, HowManyNo 

 Usage: my $only_cnt = $o_onlyno->HowManyOnly()
        my $no_cnt = $o_onlyno->HowManyNo()
 Function: Return the number of only_xxx or no_xxx parameters
 Args:  none

=cut

sub HowManyOnly
{
    my $self = shift;
    return $self->{__cnt_only};
}

sub HowManyNo
{
    my $self = shift;
    return $self->{__cnt_no};
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;

