package iANT::Entry::Display;

use AnotherTemplate;
use iANT::Entry;
use base 'iANT::Entry';
use iANT::Common;
use strict;
use Carp;



my %H_LEVELS = (0 => "0-Hypothetical", 1 => "1-Putative", 2 => "2-Probable", 3 => "3" );
#commenter le lc
my %H_CLASSIFICATION_FILES = ('riley' => 'classification_riley');


sub _Init
{
    my ($self, $source) = @_;
    $self->SUPER::_Init($source);
    $self->{'__template'} = '';
}


sub SetTemplate
{
    my ($self,$template_source) = (@_);
    $self->Set('template',$template_source);
}

sub GetTemplate
{
    my ($self) = (@_);
    $self->Get('template');
}

sub GetRawTemplate
{
    my ($self) = (@_);
    $self->Get('template');
    my $o_AnotherTemplate = New AnotherTemplate();
    $o_AnotherTemplate->Update($self->GetTemplate());
    return $o_AnotherTemplate->Get('raw_template');
}


sub Print
{
    my ($self,$method) = (@_);


    my $template = $self->GetTemplate();

    if (!defined $template || ($template eq ''))
    {
        confess ("Problem with >$template<");
    }
    else
    {
        my $o_AnotherTemplate = New AnotherTemplate();
        $o_AnotherTemplate->Update($self->GetTemplate());
    
        $o_AnotherTemplate->Set('list_separator',"\t");
        $o_AnotherTemplate->AddObject('entry',$self);
        $o_AnotherTemplate->AddObject('sequence', $self->GetSequence);
        $o_AnotherTemplate->AddObject('links',$self->GetLinks);
        
        $o_AnotherTemplate->Parse();
        my $parsed_template = $o_AnotherTemplate->Get('parsed_template');
        if ($method eq 'string')
        {
            return $parsed_template;
        }
        else
        {
            print STDOUT "$parsed_template";
        }
    }

}

=pod
	OBSOLETE !!!!!!
=cut

sub GetComment
{
    my ($self) = (@_);
    $self->SelectDefinitions({id => 'comment'});
    my $content = 'unknown';
    if  (defined ${$self->GetDefinitions}[0])
    {
        $content = ${$self->GetDefinitions}[0]->Get('content');
    }
    else
    {
        $self->Restore();
        $self->SelectDefinitions({id => 'product'});
        if  (defined ${$self->GetDefinitions}[0])
        {
            $content = ${$self->GetDefinitions}[0]->Get('content');
        }
    }
    $self->Restore();
    return $content;
}


sub GetProduct
{
    my ($self) = (@_);
    $self->SelectDefinitions({id => 'product'});
    my $content = 'unknown';
    if  (defined ${$self->GetDefinitions}[0])
    {
        $content = ${$self->GetDefinitions}[0]->Get('content');
    }
    $self->Restore();
    return $content;
}

sub GetIprProduct
{
    my ($self) = (@_);
    $self->SelectDefinitions({id => 'product', ontology => 'INTERPRO'});
    my $content = 'unknown';
    if  (defined ${$self->GetDefinitions}[0])
    {
        $content = ${$self->GetDefinitions}[0]->Get('content');
    }
    $self->Restore();
    return $content;
}

sub GetCommentConfidenceValue
{
    my ($self) = (@_);
    $self->SelectDefinitions({id => 'comment'});
    my $confidence_value = 'unknown';
    if (defined ${$self->GetDefinitions}[0])
    {
        my $confidence = ${$self->GetDefinitions}[0]->Get('confidence');
        $confidence_value = &__GetConfidenceValue($confidence);
    }
    else
    {
        $self->Restore();
        $self->SelectDefinitions({id => 'product'});
        if (defined ${$self->GetDefinitions}[0])
        {
            my $confidence = ${$self->GetDefinitions}[0]->Get('confidence');
            $confidence_value = &__GetConfidenceValue($confidence);
        }
    }
    $self->Restore();
    return $confidence_value;
}

sub CommentConfidenceHtmlSelect
{
    my ($self,$select_name) = (@_);
    $self->SelectDefinitions({id => 'comment'});
    my $confidence = '';
    if (defined ${$self->GetDefinitions}[0])
    {
        $confidence = ${$self->GetDefinitions}[0]->Get('confidence');
    }
    $self->Restore();
    return &iANT::Common::BuildHtmlSelect(\%H_LEVELS,$select_name,$confidence);
}

sub GetClassification
{
    my ($self,$classification_type) = (@_);
    $self->SelectDefinitions({ontology => "$classification_type"});
    my $content = 'unknown';
    if (defined ${$self->GetDefinitions}[0])
    {
        $content = ${$self->GetDefinitions}[0]->Get('id');
    }
    $self->Restore();
    return $content;
}


sub GetClassificationValue
{
    my ($self,$classification_type) = (@_);
    $self->SelectDefinitions({ontology => "$classification_type"});
    my $classification_content = 'unknown';

    if (defined ${$self->GetDefinitions}[0])
    {
        $classification_content = ${$self->GetDefinitions}[0]->Get('content');
        if (! defined $classification_content || ( $classification_content eq ''))
        {
            my $classification = $self->GetClassification($classification_type);
            $classification_content = &__GetClassificationValue($classification);
        }
    }
    

    $self->Restore();
    return $classification_content;
}

sub ClassificationHTMLSelect
{
    my ($self,$classification_type,$select_name) = (@_);
    $self->SelectDefinitions({ontology => "$classification_type"});
    my $classification = '';


    if (defined ${$self->GetDefinitions}[0])
    {
        $classification = ${$self->GetDefinitions}[0]->Get('id');
        $self->Restore();
    }
    return &iANT::Common::BuildHtmlSelect(\%H_LEVELS,$select_name,$classification);
}


sub __GetConfidenceValue
{
    my ($confidence) = (@_);
    
    return &iANT::Common::GetValue(\%H_LEVELS,$confidence);
}

sub __GetClassificationValue
{
    my $classification = shift;
    $classification = lc $classification;
    my $o_param;
    $o_param = &iANT::Cfg::Load($o_param);
    return &iANT::Common::GetValue($o_param->Get($H_CLASSIFICATION_FILES{$classification}),$classification);

}



1;
