#!/usr/bin/env perl

package iANT::Project::User;

use strict;
use warnings;
use XML::Twig;
use iANT::Common;
use iANT::project::user::user;


=head1 PUBLIC METHODS

=pod 

=head3 function New

 Title        : New
 Usage        : New iANT::User;
 Prerequisite : none
 Function     : Create an object iANT::User
 Returns      : An object iANT::User
 Args         : none
 Globals      : none

=cut

sub _Init
{
    my ($self, $source) = (@_);

    $self->{__login} = "";
    $self->{__level} = "";
    
    
    if (defined $source)
    {
        $self = &Update($self, $source);
    }

}


sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_entry;
    
    my $rh_param = {};
    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (ref($source) =~ /paramparser/i)
    {
        $source->Dump('HASH', $rh_param,'');
        $self = &__InitializeFromHash($self, $rh_param);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        my $o_param = New ParamParser($source);
        $o_param->Dump('HASH', $rh_param,'');
        $self = &__InitializeFromHash($self, $rh_param);
    }
    else
    {
        confess("Unknown source");
    }
    return $self;
}


=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_project->GetXml($target_element);
 Prerequisite : none
 Function     : Retrieve a User (or part of) in xml format
 Returns      : An xml string
 Args         : $target_element: optional; if empty or undef, retrieve all
                xml elements.
 Globals      : none

=cut

sub GetXml
{
    my ($self, $target_element) = @_;
    my $o_xml_user = iANT::project::user::user->new();

    if (   (!defined $target_element)
        || ($target_element eq 'user')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        $o_xml_user->login($self->{__login});
        $o_xml_user->level($self->{__level});
        
    }
    return $o_xml_user;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_entry->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump an Project in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                        -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    return &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    return &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);

}


sub Restore
{
    my ($self) = @_;
    return &iANT::Common::Restore(@_);
    
}


=item GetLogin

    Returns      : User login

=cut

sub GetLogin
{
    my ($self) = @_;
    return $self->Get('login');
}

=item SetLogin

    Set User login

=cut

sub SetLogin
{
    my ($self, $login) = @_;
    $self->Set('login', $login);
}

=item GetLevel

    Returns      : User level

=cut

sub GetLevel
{
    my ($self) = @_;
    return $self->Get('level');
}

=item SetLevel

    Set User level

=cut

sub SetLevel
{
    my ($self, $level) = @_;
    $self->Set('level', $level);
}

=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_user,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing User object from a hash (attr => value, ...)
 Returns      : An object User
 Args         : $o_user:
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    
    $self->{__login}     = $rh_attributes->{login}     if (defined $rh_attributes->{login});
    $self->{__level}     = $rh_attributes->{level}     if (defined $rh_attributes->{level});
    return $self;
}

1;
