/*
   $Id: DisplaySam.h 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __DISPLAYSAM_H__
#define __DISPLAYSAM_H__

#include <map>
using namespace std;

#include "Parameters.h"
#include "rtstack.h"
#include "Display.h"
#include "../samtools/sam.h"

class Bank;
class DisplaySAM: public Display {
	/**
	 * \brief Wrapping samopen/samclose inside a little object: open the file in the destructor, close it in the destructor
	 * \todo GetFile() il faudrait tester le type pour plus de fiabilite
	 */
	class SamFd: private NonCopyable
	{
		public:
			SamFd(): __samout(NULL) {};
			SamFd (const char *fn, const char *mode, const void *aux): __samout(NULL) {SamOpen(fn,mode,aux);};
			~SamFd() { if (__samout!=NULL && __samout->x.tamw!=stdout) samclose(__samout); };
			bool IsOpened() const { return (__samout!=NULL); };
	
			// Get the wrapped sam file and the wrapped FILE *	
			samfile_t* Get() const { return __samout; };
			FILE* GetFile() const 
			{
				if (IsOpened())
				{
					return __samout -> x.tamw; 
				}
				else
				{
					throw( rtstack("ERROR - sam file not yet opened !!!"));
				};
			};
			
			// Open a new sam file, unless already opened
			void SamOpen (const char *fn, const char *mode, const void *aux) {if (!IsOpened()) __samout=samopen(fn,mode,aux);};
	
		private:
			samfile_t* __samout;

	};
	public:
	    DisplaySAM(bool ext=false, bool mate=false);
		virtual void DisplayTopHeader() const;
	    void DisplayHsp(const Hsp&) const;
	
	protected:
		void _SetToBam() { __bam = true; };
		
	private:
		Bank& __bank;
		bool __bam;
		bool __ext;
		bool __mate;	// If true, write to the Mate output file
		const unsigned char* __bank_seq;
		mutable SamFd __samout;
		mutable map<string,int> __name2tid;

		void __CopySequence      ( const Hsp& hsp, bam1_t* ) const;
		void __WriteQual         ( const Hsp& hsp, bam1_t* ) const;
		void __WriteCigar        ( const Hsp& hsp, bam1_t* ) const;
		void __WriteAux          ( const Hsp& hsp, bam1_t* ) const;
		void __WritePairingInfos ( const Hsp& hsp, bam1_t* ) const;
		void __WriteMD           ( const Hsp& hsp, bam1_t* ) const;
};

class DisplayBAM: public DisplaySAM {
	public:
		DisplayBAM(bool ext=false, bool mate=false) : DisplaySAM(ext,mate) { _SetToBam();};
};
#endif
