#!/usr/bin/perl -I perl

# $Id: primer3InputFile.pl 648 2011-10-31 12:59:22Z manu $

=pod
=head1 NAME

primer3InputFile - This program runs Primer3 and parses the output File

=head1  SYNOPSIS

primer3InputFile.pl

=head1 DESCRIPTION

This program create a BoulderIO file from a multifasta file,
then runs Primer 3 using a system commandline
and finally it parses the outputfile and return all the informations 
on the bunch of primers

=cut

=head1 SUBROUTINES

CreateIntputFile
GetOutputInfo
DisplaySTS

=cut

use strict;
use warnings;


MAIN:
{
	
    &CreateIntputFile( "freq.tmp" );
    print "Creation of 'inputFile.tmp'\n" ;
    
    #~ system ( "/home/asagne/Travail/ProberLikeSoft/audrey_01032011/eFish/primer3-2.2.3/src/primer3_core -p3_settings_file /home/asagne/tatum/src/Primer3.param < inputFile.tmp -output output.tmp " ) ;
    system ( "/home/asagne/Travail/ProberLikeSoft/audrey_01032011/eFish/primer3-2.2.3/src/primer3_core  < inputFile.tmp -output output.tmp " ) ;
    print "Primer3 ran \nCreation of 'output.tmp'\n" ;
    
    my @a_result = &GetOutputInfo( "output.tmp" ) ;
    print "Output data saved\n" ;

	&DisplaySTS( @a_result );
	print "Creation of 'primer3Output.sts'\n";
	
	system ( "/home/asagne/tatum/eFish/ePCR/Linux-x86_32/e-PCR primer3Output.sts ssc1.fa O=ePCR.tmp" ) ;
	print "e-PCR ran\n" ;
	
	&CreateNarcisseInput( "ePCR.tmp" );
	print "Creation of 'narcisseInput.txt'\n" ;
	
}


=head2 CreateIntputFile

 Usage    : Create the BoulderIO File, input for Primer3
 Function : CreateInputFile
 Arg      : $multi_fasta_file Path of the multifasta file of Suitable zones
 Return   : void

 Title        : CreateIntputFile
 Usage        : &CreateIntputFile( $multi_fasta_file )
 Prerequisite : none
 Function     : Create the BoulderIO File in the current directory, this is the input file for Primer3
 Returns      : void
 Args         : $multi_fasta_file   Path of the multifasta file of Suitable zones
 Globals      : none

=cut
sub CreateIntputFile 
{
	my ( $multi_fasta_file ) = @_;
	
	#Get all the lines of the file
	open ( FILE, $multi_fasta_file ) || die "Erreur E/S : $!\n";
	my $all_lines   = "";
	my $line        = "";
	
	while ( $line = <FILE> )
	{
		$all_lines.= $line;
	}
	close ( FILE );
	
	#Split the multifasta file with "\n"
	my @a_sequences = split ( '\n', $all_lines );
	
	#write the inputFile with tags and values
	open( FILE, ">inputFile.tmp" ) || die "Erreur E/S:$!\n";
	
	foreach my $sequence ( @a_sequences )
	{
		if ( $sequence =~ /^>/ )
		{
			print FILE "SEQUENCE_ID=$sequence\nSEQUENCE_TEMPLATE=";
		}
		elsif ( $sequence =~ /^[ATGCN]+/ )
		{
			print FILE "$sequence";
		}
		else
		{
			print FILE "\n=\n";
		}
	}
	print FILE "\n=";
	close ( FILE );
}

sub parseFreq
{
	my ($glintfreqFile) = @_;
	
	#Get all the lines of the file
	open ( FILE, $glintfreqFile ) || die "Erreur E/S : $!\n";
	my $all_lines   = "";
	my $line        = "";
	
	while ( $line = <FILE> ) { $all_lines.= $line; }
	close ( FILE );
	
	#Split the multifasta file with "\n"
	my @a_sequences = split ( '\n', $all_lines );
	my $hash;
	my $id;
	
	#write the inputFile with tags and values
	#and fill the hash in the same time
	open( FILE, ">inputFile.tmp" ) || die "Erreur E/S:$!\n";	
	foreach my $sequence ( @a_sequences )
	{
		if ( $sequence =~ /^>(\d+)|(\S+)\tstart : (\d+)\tend : (\d+)/ )
		{
			   $id     = $1;
			my $header = $2;
			my $start  = $3;
			my $end    = $4;			
			if ( defined( $$hash{$id}{ID} ) )
			{
				Carp::croak( "Error: sequence $id already loaded\n" );
			}
			$$hash{$id}{HEADER} = $header;
			$$hash{$id}{START}  = $start;
			$$hash{$id}{END}    = $end;
			print FILE "SEQUENCE_ID=$sequence\nSEQUENCE_TEMPLATE=";
		}
		elsif ( $sequence =~ /^[ATGCN]+/ )
		{
			$$hash{$id}{SEQ}    = $sequence;
			print FILE "$sequence";
		}
		else
		{
			print FILE "\n=\n";
		}
	}
	print FILE "\n=";
	close ( FILE );
	return $hash;
}

=head2 GetOutputInfo

 Usage    : Parses the outputFile of Primer3
 Function : GetOutputInfo
 Arg      : $output_file Path of the output file of Primer3
 Return   : $result an array of all the sequences and the primers corresponding

 Title        : GetOutputInfo
 Usage        : @result = &GetOutputInfo( $output_file )
 Prerequisite : none
 Function     : Parses the outputFile of Primer3 and return the informatinos as an array
 Returns      : an array of all the sequences and the primers corresponding
 Args         : $output_file  array of arrays contaning the informations on every pair of primers 
 Globals      : none

=cut
sub GetOutputInfo 
{
	my ( $output_file ) = @_;

	#Get all the lines of the file
	open ( FILE, $output_file ) || die "Erreur E/S : $!\n";
	my $all_lines   = "";
	my $line        = "";
	while( $line = <FILE> ) { $all_lines .= $line ; }
	close ( FILE );
	
	my @a_result ;
	my @a_seq ; 
	
	#Split the different sequences with "="
	@a_seq = split (/=\n/, $all_lines) ;
	print "Nb of fasta sequence : ", $#a_seq+1,"\n";

	foreach my $seq ( @a_seq )
	{	
		my @a_sequence_info ;
		my @a_line         = split (/\n/, $seq) ;
		my $id             = "" ;
		my $seq            = "" ;
		my $nb_pair        = "" ;
		my @a_pairs ;

		if ( $a_line[0] =~ /SEQUENCE_ID=(.*)/ )
		{
			$id = $1. "\n" ;
			push ( @a_sequence_info, $id ) ;
		}
		
		if ( $a_line[1] =~ /SEQUENCE_TEMPLATE=(.*)/ )
		{
			$seq = $1. "\n" ;
			push ( @a_sequence_info, $seq );
		}
		
		if ( $a_line[5] =~ /PRIMER_PAIR_NUM_RETURNED=(.*)/ )
		{
			$nb_pair = $1. "\n" ;
			push ( @a_sequence_info, $nb_pair );
		}

		for ( my $i = 6 ; $i < $#a_line ; $i += 20 ) 
		{
			my $pair                 = "";
			my $pair_penalty         = "";
			my $primer_right_seq     = "";
			my $primer_left_seq      = "";
			my $primer_left_start    = "";
			my $primer_left_length   = "";
			my $primer_right_start   = "";
			my $primer_right_length  = "";
			my $primer_right_tm      = "";
			my $primer_left_tm       = "";
			my $primer_right_gc      = "";
			my $primer_left_gc       = "";
			my $pair_product_size    = "";
			
			for ( my $j = $i ; $j <= $i + 19 ; $j++ )
			{
				if ( $a_line[$j] =~ /PRIMER_PAIR_\d+_PENALTY=(.*)/)
				{
					$pair_penalty = $1 ;
					$pair .= $pair_penalty."\t";
				}
				if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_SEQUENCE=(.*)/)
				{
					$primer_right_seq = $1 ;
					$pair .= $primer_right_seq."\t";
				}
				if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_SEQUENCE=(.*)/)
				{
					$primer_left_seq = $1 ;
					$pair .= $primer_left_seq. "\t";
				}
				if ( $a_line[$j] =~ /PRIMER_LEFT_\d+=(\d+),(\d+)/)
				{
					$primer_left_start = $1 ;
					$primer_left_length = $2 ;	
					$pair .= $primer_left_start. "\t". $primer_left_length. "\t";
				}
				if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+=(\d+),(\d+)/)
				{
					$primer_right_start = $1 ;
					$primer_right_length = $2 ;	
					$pair .= $primer_right_start. "\t". $primer_right_length. "\t";
				}
				if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_TM=(\d+),(\d+)/)
				{
					$primer_right_tm = $1 ;
					$pair .= $primer_right_tm. "\t";
				}
				if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_TM=(.*)/)
				{
					$primer_left_tm= $1 ;
					$pair .= $primer_left_tm. "\t";
				}
				if ( $a_line[$j] =~ /PRIMER_RIGHT_\d_GC_PERCENT=(.*)/)
				{
					$primer_right_gc = $1 ;
					$pair .= $primer_right_gc. "\t";
				}
				if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_GC_PERCENT=(.*)/)
				{
					$primer_left_gc = $1 ;
					$pair .= $primer_left_gc. "\t";
				}
				if ( $a_line[$j] =~ /PRIMER_PAIR_\d_PRODUCT_SIZE=(.*)/)
				{
					my $pair_product_size = $1 ;
					$pair .= $pair_product_size. "\n";
				}
			}
			push( @a_pairs, $pair );
		}	
		#~ print @a_pairs;
		push( @a_sequence_info, @a_pairs ) ;	
		push( @a_result, @a_sequence_info );
	}
	#~ print @a_result;
	return @a_result;

}

=head2 DisplaySTS

 Usage    : Create the UniSTS File for ePCR soft
 Function : DisplaySTS
 Arg      : @a_result Result array from Primer3 Output
 Return   : void

 Title        : DisplaySTS
 Usage        : &DisplaySTS( @a_result )
 Prerequisite : none
 Function     : Create the BoulderIO File in the current directory, this is the input file for Primer3
 Returns      : void
 Args         : $multi_fasta_file   Path of the multifasta file of Suitable zones
 Globals      : none

=cut
sub DisplaySTS
{
	my ( @a_result ) = @_;
	
	#write the inputFile with tags and values
	open( FILE, ">primer3Output.sts" ) || die "Erreur E/S:$!\n";
	
	my $nb_pair   = $a_result[2];
	my $seq_id    = 0;
	my $pair_id   = 0;
	
	print "taille du tableau result : ", $#a_result,"\n";
	
	for ( my $i = 0 ; $i <= $#a_result ; $i = ($i+3+$nb_pair))
	{
		my $pair_id   = 0;
		
		for ( my $j = $i ; $j <= $i + $nb_pair+2 ; $j++ )
		{
			if ( $a_result[$j] =~ /^.+\t(\D+)\t(\D+)\t.+\t(\d+)\n/ )
			{
				print FILE "$seq_id$pair_id\t$1\t$2\t$3\n";
				$pair_id++;
			}
		}
		$seq_id++;
		$nb_pair=$a_result[$i+2];
	}
	print FILE "\n";
	close ( FILE );
}

=head2 DisplaySTS

 Usage    : Create a file input for narcisse karyotype display
 Function : CreateNarcisseInput
 Arg      : $file Path of the ePCR result file.
 Return   : void

 Title        : CreateNarcisseInput
 Usage        : &CreateNarcisseInput( $file )
 Prerequisite : none
 Function     : Create a file corresponding to the csv data file format used as an input for Narcisse 
 Returns      : void
 Args         : $file   Path of the ePCR result file.
 Globals      : none

=cut
sub CreateNarcisseInput
{
	my ( $file ) = @_;
	
	#Get all the lines of the file
	open ( FILE, $file ) || die "Erreur E/S : $!\n";
	my $all_lines   = "";
	my $line        = "";
	
	while ( $line = <FILE> )
	{
		$all_lines.= $line;
	}
	close ( FILE );
	
	my @a_color = ( "black", "blue", "brown", "olive", "indigo", "yellowgreen", "red", "darkcyan", "gold", "green", "pink", "orange", "skyblue", "stateblue");
	my @a_sequence = split ( '\n', $all_lines );
	#write the inputFile with tags and values
	open( FILE, ">narcisseInput.txt" ) || die "Erreur E/S:$!\n";
	
	#Split the file with "\n"
	my $i = 0 ;

	foreach my $a_sequence ( @a_sequence )
	{
		if ( $a_sequence =~ /.+\s+(\d+)\.\.(\d+)\s+(\d+)(\d+)/ )
		{
			my $seq_id  = $3;
			my $pair_id = $4; 
			print FILE "1\t$seq_id$pair_id\t$1\t$2\t$a_color[$i]\n";
			$i++;
		}
	}
	close ( FILE );
}
