/*
   $Id: sizes.cpp 649 2011-10-31 13:57:42Z manu $
*/


/*
  $Id: sizes.cpp 649 2011-10-31 13:57:42Z manu $
*/

#include <iostream>
#include <limits>
using namespace std;

#include "ptypes.h"
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/mman.h>

/*

Function: Maxsize
Parameters:
Return value:

*/

void SysDie (int rvl, const char *s)
{
    if (rvl==-1)
    {
        perror(s);
        exit(1);
    };
};

size_t MaxSize(int part)
{
    size_t max_size;
    numeric_limits<ssize_t> limit_ssize_t;

    struct rlimit memory_limit;

    int rvl = getrlimit(RLIMIT_DATA,&memory_limit);
    SysDie(rvl,"MaxSize");

    if (memory_limit.rlim_cur==RLIM_INFINITY) /* No limit specified: try to know the total physical memory */
    {
        long page_size   = sysconf(_SC_PAGESIZE);
        long nb_of_pages = sysconf(_SC_PHYS_PAGES);
        long physical_memory = nb_of_pages / 100;
        physical_memory *= page_size;
        if (physical_memory < limit_ssize_t.max()/part)
        {
            physical_memory *= part;
        }
        else
        {
            physical_memory = limit_ssize_t.max();
        }
        max_size = physical_memory;
    }
    else
    {
        max_size = memory_limit.rlim_cur/100;
        max_size *= part;
    };

    /* Now, we ask the system to give us those pages */

    void * base = malloc(max_size);
    if (base==0) rvl=-1;
    SysDie(rvl,"MaxSize");
    return max_size;
};

int main(int argc,char* argv[])
{
	system("uname -a");

    cout << "\n----\n";
    cout << "int8_t\n";
    cout << "----\n";
	cout << "size:   " << sizeof(int8_t) << '\n';
    numeric_limits<int8_t> limit_int8_t;
    cout << "signed: " << (limit_int8_t.is_signed ? "Yes" : "No ") << "\n";
    cout << "min:    " << (int) limit_int8_t.min() << '\n';
    cout << "max:    " << (int) limit_int8_t.max() << '\n';

    cout << "\n----\n";
    cout << "int16_t\n";
    cout << "----\n";
	cout << "size:   " << sizeof(int16_t) << '\n';
    numeric_limits<int16_t> limit_int16_t;
    cout << "signed: " << (limit_int16_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << limit_int16_t.min() << '\n';
    cout << "max:    " << limit_int16_t.max() << '\n';

    cout << "\n----\n";
    cout << "int32_t\n";
    cout << "----\n";
	cout << "size:   " << sizeof(int32_t) << '\n';
    numeric_limits<int32_t> limit_int32_t;
    cout << "signed: " << (limit_int32_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << limit_int32_t.min() << '\n';
    cout << "max:    " << limit_int32_t.max() << '\n';

    cout << "\n-----\n";
    cout << "uint8_t\n";
    cout << "-----\n";
	cout << "size:   " << sizeof(uint8_t) << '\n';
    numeric_limits<uint8_t> limit_uint8_t;
    cout << "signed: " << (limit_uint8_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << (int) limit_uint8_t.min() << '\n';
    cout << "max:    " << (int) limit_uint8_t.max() << '\n';

    cout << "\n-----\n";
    cout << "uint16_t\n";
    cout << "-----\n";
	cout << "size:   " << sizeof(uint16_t) << '\n';
    numeric_limits<uint16_t> limit_uint16_t;
    cout << "signed: " << (limit_uint16_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << limit_uint16_t.min() << '\n';
    cout << "max:    " << limit_uint16_t.max() << '\n';

    cout << "\n-----\n";
    cout << "uint32_t\n";
    cout << "-----\n";
	cout << "size:   " << sizeof(uint32_t) << '\n';
    numeric_limits<uint32_t> limit_uint32_t;
    cout << "signed: " << (limit_uint32_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << limit_uint32_t.min() << '\n';
    cout << "max:    " << limit_uint32_t.max() << '\n';

    cout << "\n-----\n";
    cout << "uint64_t\n";
    cout << "-----\n";
	cout << "size:   " << sizeof(uint64_t) << '\n';
    numeric_limits<uint64_t> limit_uint64_t;
    cout << "signed: " << (limit_uint64_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << limit_uint64_t.min() << '\n';
    cout << "max:    " << limit_uint64_t.max() << '\n';

    cout << "\n-----\n";
    cout << "ssize_t\n";
    cout << "-----\n";
	cout << "size:   " << sizeof(ssize_t) << '\n';
    numeric_limits<ssize_t> limit_ssize_t;
    cout << "signed: " << (limit_ssize_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << limit_ssize_t.min() << '\n';
    cout << "max:    " << limit_ssize_t.max() << '\n';

    cout << "\n-----\n";
    cout << "size_t\n";
    cout << "-----\n";
	cout << "size:   " << sizeof(size_t) << '\n';
    numeric_limits<size_t> limit_size_t;
    cout << "signed: " << (limit_size_t.is_signed ? "Yes" : "No") << '\n';
    cout << "min:    " << limit_size_t.min() << '\n';
    cout << "max:    " << limit_size_t.max() << '\n';

    cout << "\n-----\n";
    cout << "SSIZE_MAX " << SSIZE_MAX << "\n";

    long page_size   = sysconf(_SC_PAGESIZE);
    long nb_of_pages = sysconf(_SC_PHYS_PAGES);
    double physical_memory = (double) page_size * (double) nb_of_pages;
    cout << "\n-----\n";
    cout << "Page size       " << page_size << endl;
    cout << "nb of pages     " << nb_of_pages << endl;
    cout << "physical memory  (b) " << physical_memory << endl;
    cout << "physical memory  (Kb)" << physical_memory/1024 << endl;
    cout << "physical memory  (Mb)" << physical_memory/(1024*1024) << endl;

    cout << "\n-----\n";
    cout << "Now asking a lot of memory\n";
    size_t mem = MaxSize(50);
    cout << "Got " << mem << " Bytes\n";
    sleep(10);
	return 0;
}



