#! /usr/bin/perl

# $Id: test.pl 640 2011-10-24 13:44:12Z manu $

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

use strict;
use warnings;
use File::Spec;
use File::Basename;

# Finding some perl modules distributed with glint 
use FindBin;
use lib "$FindBin::Bin/perl";
use General;
use Sufine;
use ParamParser;

use constant GLINT_OPT_DEF      => "";
use constant GLINTINDEX_OPT_DEF => "-C 0";
use constant TIME_OPT           => "-f '%E %S %U %K %M %F'";
use constant GLINT_EXE          => 'glint';
#use constant GLINT_EXE          => '/home/manu/devt-local/glint/src/glint';
use constant GLINTINDEX_EXE     => 'glint';
#use constant GLINTINDEX_EXE     => '/home/manu/devt-local/glint/src/glintindex';
use constant TIME_EXE           => '/usr/bin/time';
use constant GLINT_COMMAND      => "align";
use constant GLINT_SELF_COMMAND => "self";

MAIN:
{
	ParamParser::SetDefaultBehaviour('exit_on_getopt_error');
	my $o_param=New ParamParser('GETOPTLONG',\&Usage,'run','compare','data_directory=s','reference_directory=s','target_directory=s','bank=s','request=s','ncpus=i',
	                                                 'self','pipe','check_svn','auto_index','glint_command=s','glintindex_opt=s','glint_opt=s','glint_exe=s','glintindex_exe=s','only_output','o_switch','help');
	
	# Check parameters, default values
	my $compare_flg;	
	if ( $o_param -> IsDefined ( 'run' ) && $o_param -> IsDefined ( 'compare' ) )
	{
		&Usage();
		die "ERROR - ONLY ONE of the switches 'run' or 'compare' must be specified\n";
	}
	if ( $o_param -> IsDefined ( 'run' ) )
	{
		$compare_flg = &FALSE;
		$o_param->AssertDirExists('data_directory');
		$o_param->AssertDefined('target_directory');
		-x &TIME_EXE || die &TIME_EXE . " not found or not executable";
	}
	elsif ( $o_param -> IsDefined ( 'compare' ) )
	{
		$compare_flg = &TRUE;
		$o_param->AssertDirExists('target_directory');
		$o_param->AssertDirExists('reference_directory');
	}
	else
	{
		&Usage();
		die "ERROR - One of the switches 'run' or 'compare' MUST be specified\n";
	}
	my $self_flg       = $o_param -> IsDefined('self');
	my $auto_index_flg = $o_param -> IsDefined('auto_index');
	my $o_switch_flg   = $o_param -> IsDefined('o_switch');

	my $pipe_flg       = $o_param -> IsDefined('pipe');
	
	if ( $self_flg && $pipe_flg)
	{
		die "ERROR - Only one of self or pipe must be specified.\n";
	}
		
	$o_param->SetUnlessDefined('glint_opt',&GLINT_OPT_DEF);
	if ( $o_param -> IsDefined ( 'ncpus' ) )
	{
		$o_param -> AssertInteger('ncpus');
		$o_param->Set( 'glint_opt', $o_param -> Get ( 'glint_opt') . ' -a ' . $o_param -> Get ( 'ncpus' ) );
	}
	$o_param->SetUnlessDefined('glintindex_opt',&GLINTINDEX_OPT_DEF);
	$o_param->SetUnlessDefined('bank','bank.fasta');
	$o_param->SetUnlessDefined('request','request.fasta');
	if ( $o_param -> IsDefined('self') )
	{
		$o_param->SetUnlessDefined('glint_command',&GLINT_SELF_COMMAND);
	}
	else
	{
		$o_param->SetUnlessDefined('glint_command',&GLINT_COMMAND);
	}
	$o_param->AssertAllowedValue('glint_command','index','align','self','map','mappe','chain','stat','convert','slice','freq');
	
	my $sav_dir    = File::Spec->rel2abs(File::Spec->curdir());
	$o_param -> SetUnlessDefined('glintindex_exe',&GLINTINDEX_EXE);
	unless (File::Spec->file_name_is_absolute( $o_param->Get('glintindex_exe') ) )
	{
		$o_param -> Set ( 'glintindex_exe',File::Spec->catfile($sav_dir,$o_param -> Get('glintindex_exe') ) );
	};
	-x $o_param->Get('glintindex_exe') || die $o_param->Get('glintindex_exe') . " not found or not executable";
	$o_param->Set( 'glintindex_exe', $o_param -> Get('glintindex_exe') . " index " );
	
	$o_param -> SetUnlessDefined('glint_exe',&GLINT_EXE);
	unless (File::Spec->file_name_is_absolute( $o_param->Get('glint_exe') ) )
	{
		$o_param -> Set ( 'glint_exe',File::Spec->catfile($sav_dir,$o_param -> Get('glint_exe') ) );
	};
	-x $o_param->Get('glint_exe') || die $o_param->Get('glint_exe') . " not found or not executable";
	$o_param->Set( 'glint_exe', $o_param -> Get('glint_exe') . " " . $o_param -> Get('glint_command') );

	my $ref_dir     = File::Spec->rel2abs($o_param->Get('reference_directory'));
	my $bank        = $o_param->Get('bank');
	my $data        = $o_param->Get('data');
	
	# computing and may be creating the target directory
	my $target_dir  = File::Spec->rel2abs($o_param->Get('target_directory'));
	if ( $o_param->IsDefined('check_svn') && $o_param->GetInteger('check_svn') == &TRUE)
	{
		my $rev = &CheckRelease();
		$target_dir = File::Spec->catfile($target_dir,$rev);
		warn "SVN revision = $rev\n";
	}
	
	if ( -e $target_dir && ! -d $target_dir )
	{
		die "ERROR - $target_dir should not exist OR should be a directory\n";
	}
	mkdir $target_dir if ( ! -e $target_dir);
	    
	# If $compare_flg is false, we run glintindex, then glint
	if ($compare_flg==&FALSE)
	{
		if ( $self_flg==&TRUE)
		{
			if ( $auto_index_flg==&TRUE)
			{
				&RunGlintAGSO( $o_param, $target_dir);
			}
			else
			{
				&RunGlintIGSO( $o_param, $target_dir);
			}
		}
		elsif ( $auto_index_flg==&TRUE )
		{
			if ( $o_switch_flg==&TRUE )
			{
				&RunGlintAGO( $o_param, $target_dir);
			}
			else
			{
				&RunGlintAG( $o_param, $target_dir);
			}
		}
		else
		{
			&RunGlintIG( $o_param, $target_dir);
		}
	}
	else
	{
		&CompareResults( $o_param, $target_dir );
	}
}

=pod

=head2 Function Cat

 Usage:     &Cat($fh_out,$filename)
 Procedure: print every line of $filenmae to $fh_out
 Arg :      $fh_out   An opened file handle, defaults to stdout
            $filename A file name

=cut
sub Cat
{
	my ( $fh_out, $file ) = @_;
	$fh_out = *STDOUT unless defined $fh_out;
	
	my $fh_in = new IO::File $file;
	while (my $line = <$fh_in>)
	{
		print $fh_out $line;
	}
}

=pod

=head2 Function ComputePrefix

 Usage:     my $prefix = &ComputePrefix( $dir );
 Function:  From the directory passed by parameter, taking its name and its directory name,
            compute a prefix that will be used in filenames
 Arg :      $dir (optional)   The directory to use for computing the prefix
                              The current directory is used by default
 Return:    The prefix

=cut
sub ComputePrefix
{
    my $dir    = shift;
    $dir       = File::Spec->rel2abs(File::Spec->curdir()) unless (defined $dir);
    my $prefix = basename(dirname($dir)).'_'.basename($dir);
	return $prefix;
}

=pod

=head2 Function CheckRelease

 Usage:     &CheckRelease
 Function:  Does work only if the perl package SVN is installed (protected by eval however)
			Calls svn status on the .. directory, toknow if there is anything to commit
			Dies if there are files with status Modified, Added or Conflicted
			Else, calls svn info and returns the revision number
 Arg :      none
 Return:    Release number

=cut

sub CheckRelease
{
	eval "use SVN::Client";
	eval "use SVN::Wc:Status";
	
	my $svn = new SVN::Client();
	my (@a_conflicted,@a_added,@a_modified,@a_other);
	no warnings;
	my $status_func = sub {
		my ( $path, $status) = @_;
		#print "coucou $path " . $status->text_status() ."\n";
		push (@a_conflicted, $path), return if ( $status->text_status() == $SVN::Wc::Status::conflicted );
		push (@a_modified, $path), return if ( $status->text_status() == $SVN::Wc::Status::modified );
		push (@a_added, $path), return if ( $status->text_status() == $SVN::Wc::Status::added );

	};
	use warnings;
	$svn->status('..','WORKING',$status_func,1,0,0,0);
	
	# If there are some items with the status 'Added'/'Modified'/'conflicted', we give up
	my $flg_exit = &FALSE;
	foreach my $path (@a_conflicted)
	{
		print "C $path\n";
		$flg_exit = &TRUE;
	}
	foreach my $path (@a_modified)
	{
		print "M $path\n";
		$flg_exit = &TRUE;
	}
	foreach my $path (@a_added)
	{
		print "A $path\n";
		$flg_exit = &TRUE;
	}
	foreach my $path (@a_other)
	{
		print "$path\n";
	}
	if ($flg_exit==&TRUE)
	{
		die "ERROR - Please commit before calling the --check_svn switch";
	}
	
	my $rev;
	my $receiver = sub {
		                my( $path, $info, $pool ) = @_;
		                $rev = $info->rev;
					};
    $svn->info( '..', undef, 'WORKING', $receiver, 0 );
    return $rev;
};

=pod

=head2 Procedure RunGlintIG

 Usage:     &RunGlintIG($o_param, $target_dir);
 Function:  Run glint, ie: 1/ run glintindex to index to bank
                           2/ run glint to compare the request against the bank
 Arg :      $o_param, $target_dir

=cut
sub RunGlintIG
{
	my ($o_param,$target_dir) = @_;

	my $sav_dir    = File::Spec->rel2abs(File::Spec->curdir());
	my $data_dir    = File::Spec->rel2abs($o_param->Get('data_directory'));
	my $path_bank   = File::Spec->file_name_is_absolute( $o_param->Get('bank') ) ? 
						$o_param->Get('bank')
						:
						File::Spec->catfile($data_dir,$o_param->Get('bank'));
	my ($b1,$b2,$bank) = File::Spec->splitpath( $path_bank );
	
	my $path_request= File::Spec->file_name_is_absolute( $o_param->Get('request') ) ? 
						$o_param->Get('request')
						:
						File::Spec->catfile($data_dir,$o_param->Get('request'));
	my ($r1,$r2,$request) = File::Spec->splitpath( $path_request );
    
    my $time_exe       = &TIME_EXE . ' ' . &TIME_OPT;

	my $glintindex_exe = $o_param -> Get ( 'glintindex_exe' );	
	my $glint_exe      = $o_param -> Get ( 'glint_exe' );

	my $glintindex_opt = $o_param->Get('glintindex_opt');
	my $glint_opt      = $o_param->Get('glint_opt');
	my $pipe_flg       = $o_param -> IsDefined( 'pipe');

	chdir $target_dir;
	warn "Now working in directory $target_dir\n";

    # Create a symlink to the bank and request, avoiding a copy
	symlink ( $path_bank, $bank       );
	symlink ( $path_request, $request );	

    my $prefix    = &ComputePrefix();
    my $file_time = Sufine::GetFileName(-name => ">${prefix}_time",-ext => 'time');
    my $fh_out    = New Sufine(-name => ">${prefix}_out",-ext => 'out');
    my $file_out  = $fh_out->GetFileName();
    print $fh_out "TESTING glintindex and glint - standard output\n";
    print $fh_out "glintindex = $glintindex_exe $glintindex_opt\n";
    print $fh_out "glint      = $glint_exe $glint_opt\n";
    print $fh_out "BANK       = $bank\n";
    print $fh_out "REQUEST    = $request\n";
    
    my $fh_err   = New Sufine(-name => ">${prefix}_err",-ext => 'err');
    my $file_err = $fh_err->GetFileName();
    print $fh_err "TESTING glintindex and glint - standard error\n";
    print $fh_err "glintindex = $glintindex_exe $glintindex_opt\n";
    print $fh_err "glint      = $glint_exe $glint_opt\n";
    print $fh_err "BANK       = $bank\n";
    print $fh_err "REQUEST    = $request\n";
    
    my ($cmd,$msg);
    # Executing glintindex
    $cmd = "$glintindex_exe $glintindex_opt $bank >>$file_out 2>>$file_err";
    $msg = "STARTING glintindex\n=====================\n$cmd\n";
    print $msg;
    print $fh_out $msg;
    print $fh_err $msg;
    $fh_out -> close();
	$fh_err -> close();
    system("$time_exe --output=$file_time --append $cmd");
    
    # Executing glint
    # Executing glint
    if ( $pipe_flg )
    {
		$cmd = "cat $request | $glint_exe $glint_opt $bank '-' >>$file_out 2>>$file_err"; 
		$msg = "STARTING glint\n================\n$cmd\n";
		$cmd = "cat $request | $time_exe --output=$file_time --append $glint_exe $glint_opt $bank '-' >>$file_out 2>>$file_err"; 
	}
	else
	{
	    $cmd = "$glint_exe $glint_opt $bank $request >>$file_out 2>>$file_err"; 
		$msg = "STARTING glint\n================\n$cmd\n";	    
	    $cmd = "$time_exe --output=$file_time --append $cmd"; 
	}
    $fh_out = new IO::File $file_out,'a';
    $fh_err = new IO::File $file_err,'a';
    print $msg;
    print $fh_out $msg;
    print $fh_err $msg;
    $fh_out -> close();
 	$fh_err -> close();
	system("$cmd");
  
    chdir $sav_dir;
}

=head2 Procedure RunGlintAG

 Usage:     &RunGlintAG($o_param, $target_dir);
 Function:  Run glint in autoindex mode
 Arg :      $o_param, $target_dir

=cut
sub RunGlintAG
{
	my ($o_param,$target_dir) = @_;

	my $sav_dir    = File::Spec->rel2abs(File::Spec->curdir());
	my $data_dir    = File::Spec->rel2abs($o_param->Get('data_directory'));
	my $path_bank   = File::Spec->file_name_is_absolute( $o_param->Get('bank') ) ? 
						$o_param->Get('bank')
						:
						File::Spec->catfile($data_dir,$o_param->Get('bank'));
	my ($b1,$b2,$bank) = File::Spec->splitpath( $path_bank );
	
	my $path_request= File::Spec->file_name_is_absolute( $o_param->Get('request') ) ? 
						$o_param->Get('request')
						:
						File::Spec->catfile($data_dir,$o_param->Get('request'));
	my ($r1,$r2,$request) = File::Spec->splitpath( $path_request );
    
    my $time_exe       = &TIME_EXE . ' ' . &TIME_OPT;
	my $glint_exe      = $o_param -> Get ( 'glint_exe' );

	my $glint_opt      = $o_param->Get('glint_opt');
	my $pipe_flg       = $o_param -> IsDefined( 'pipe');
	
	chdir $target_dir;
	warn "Now working in directory $target_dir\n";

    # Create a symlink to the bank and request, avoiding a copy
	symlink ( $path_bank, $bank       );
	symlink ( $path_request, $request );	

    my $prefix    = &ComputePrefix();
    my $file_time = Sufine::GetFileName(-name => ">${prefix}_time",-ext => 'time');
    my $fh_out    = New Sufine(-name => ">${prefix}_out",-ext => 'out');
    my $file_out  = $fh_out->GetFileName();
    print $fh_out "TESTING glint in autoindex mode - standard output\n";
    print $fh_out "glint      = $glint_exe $glint_opt\n";
    print $fh_out "BANK       = $bank\n";
    print $fh_out "REQUEST    = $request\n";
    
    my $fh_err   = New Sufine(-name => ">${prefix}_err",-ext => 'err');
    my $file_err = $fh_err->GetFileName();
    print $fh_err "TESTING glint in autoindex mode - standard error\n";
    print $fh_err "glint      = $glint_exe $glint_opt\n";
    print $fh_err "BANK       = $bank\n";
    print $fh_err "REQUEST    = $request\n";
    
    my ($cmd,$msg);
    # Executing glint
    if ( $pipe_flg )
    {
		$cmd = "cat $request | $glint_exe $glint_opt $bank '-' >>$file_out 2>>$file_err"; 
		$msg = "STARTING glint\n================\n$cmd\n";
		$cmd = "cat $request | $time_exe --output=$file_time --append $glint_exe $glint_opt $bank '-' >>$file_out 2>>$file_err"; 
	}
	else
	{
	    $cmd = "$glint_exe $glint_opt $bank $request >>$file_out 2>>$file_err"; 
	    $msg = "STARTING glint\n================\n$cmd\n";
	    $cmd = "$time_exe --output=$file_time --append $cmd"; 
	}
	
    $fh_out = new IO::File $file_out,'a';
    $fh_err = new IO::File $file_err,'a';
    print $msg;
    print $fh_out $msg;
    print $fh_err $msg;
    $fh_out -> close();
 	$fh_err -> close();
    system("$cmd");
    
    chdir $sav_dir;
}

=head2 Procedure RunGlintAGO

 Usage:     &RunGlintAG($o_param, $target_dir);
 Function:  Run glint in autoindex mode with the -o switch
 Arg :      $o_param, $target_dir

=cut
sub RunGlintAGO
{
	my ($o_param,$target_dir) = @_;

	my $sav_dir    = File::Spec->rel2abs(File::Spec->curdir());
	my $data_dir    = File::Spec->rel2abs($o_param->Get('data_directory'));
	my $path_bank   = File::Spec->file_name_is_absolute( $o_param->Get('bank') ) ? 
						$o_param->Get('bank')
						:
						File::Spec->catfile($data_dir,$o_param->Get('bank'));
	my ($b1,$b2,$bank) = File::Spec->splitpath( $path_bank );
	
	my $path_request= File::Spec->file_name_is_absolute( $o_param->Get('request') ) ? 
						$o_param->Get('request')
						:
						File::Spec->catfile($data_dir,$o_param->Get('request'));
	my ($r1,$r2,$request) = File::Spec->splitpath( $path_request );
    
    my $time_exe       = &TIME_EXE . ' ' . &TIME_OPT;
	my $glint_exe      = $o_param -> Get ( 'glint_exe' );
    
	my $glint_opt      = $o_param->Get('glint_opt');
	my $pipe_flg       = $o_param -> IsDefined( 'pipe');
	
	chdir $target_dir;
	warn "Now working in directory $target_dir\n";

    # Create a symlink to the bank and request, avoiding a copy
	symlink ( $path_bank, $bank       );
	symlink ( $path_request, $request );	

    my $prefix    = &ComputePrefix();
    my $file_time = Sufine::GetFileName(-name => ">${prefix}_time",-ext => 'time');
    my $fh_out    = New Sufine(-name => ">${prefix}_out",-ext => 'out');
    my $file_out  = $fh_out->GetFileName();
    $fh_out -> close();
    
    my $fh_err   = New Sufine(-name => ">${prefix}_err",-ext => 'err');
    my $file_err = $fh_err->GetFileName();
    print $fh_err "TESTING glint in autoindex mode - standard error\n";
    print $fh_err "glint      = $glint_exe $glint_opt\n";
    print $fh_err "BANK       = $bank\n";
    print $fh_err "REQUEST    = $request\n";
    
    my ($cmd,$msg);
    # Executing glint
    if ( $pipe_flg )
    {
		$cmd = "cat $request | $glint_exe $glint_opt $bank '-' -o $file_out 2>>$file_err"; 
		$msg = "STARTING glint\n================\n$cmd\n";
		$cmd = "cat $request | $time_exe --output=$file_time --append $glint_exe $glint_opt $bank '-' -o $file_out 2>>$file_err"; 
	}
	else
	{
	    $cmd = "$glint_exe $glint_opt $bank $request -o $file_out 2>>$file_err"; 
	    $msg = "STARTING glint\n================\n$cmd\n";
	    $cmd = "$time_exe --output=$file_time --append $cmd"; 
	}
	
    $fh_err = new IO::File $file_err,'a';
    print $msg;
    print $fh_err $msg;
 	$fh_err -> close();
    system("$cmd");
    
    chdir $sav_dir;
}
    
=pod

=head2 Procedure RunGlintIGSO

 Usage:     &RunGlintIGS($o_param, $target_dir);
 Function:  Run glint, ie: 1/ run glintindex to index to bank
                           2/ run glint self to compare the bank against itself
 Arg :      $o_param, $target_dir

=cut
sub RunGlintIGSO
{
	my ($o_param,$target_dir) = @_;

	my $sav_dir    = File::Spec->rel2abs(File::Spec->curdir());
	my $data_dir    = File::Spec->rel2abs($o_param->Get('data_directory'));
	my $path_bank   = File::Spec->file_name_is_absolute( $o_param->Get('bank') ) ? 
						$o_param->Get('bank')
						:
						File::Spec->catfile($data_dir,$o_param->Get('bank'));
	my ($b1,$b2,$bank) = File::Spec->splitpath( $path_bank );
	
    my $time_exe       = &TIME_EXE . ' ' . &TIME_OPT;
   	my $glintindex_exe = $o_param -> Get ( 'glintindex_exe' );	
	my $glint_exe      = $o_param -> Get ( 'glint_exe' );

	my $glintindex_opt = $o_param->Get('glintindex_opt');
	my $glint_opt      = $o_param->Get('glint_opt');
	
	chdir $target_dir;
	warn "Now working in directory $target_dir\n";

    # Create a symlink to the bank avoiding a copy
	symlink ( $path_bank, $bank       );

    my $prefix    = &ComputePrefix();
    my $file_time = Sufine::GetFileName(-name => ">${prefix}_time",-ext => 'time');
    my $fh_out    = New Sufine(-name => ">${prefix}_out",-ext => 'out');
    $fh_out -> close();
    my $file_out  = $fh_out->GetFileName();
    $glint_opt .= " -o $file_out";
    
    my $fh_err   = New Sufine(-name => ">${prefix}_err",-ext => 'err');
    my $file_err = $fh_err->GetFileName();
    print $fh_err "TESTING glintindex and glint SELF MODE - standard error\n";
    print $fh_err "glintindex = $glintindex_exe $glintindex_opt\n";
    print $fh_err "glint      = $glint_exe $glint_opt\n";
    print $fh_err "BANK       = $bank\n";
    
    my ($cmd,$msg);

    # Executing glintindex
    $cmd = "$glintindex_exe $glintindex_opt $bank 2>>$file_err";
    $msg = "STARTING glintindex\n=====================\n$cmd\n";
    print $msg;
    print $fh_err $msg;
	$fh_err -> close();
    system("$time_exe --output=$file_time --append $cmd");
    
    # Executing glint
    $cmd = "$glint_exe $glint_opt $bank 2>>$file_err"; 
    $msg = "STARTING glint\n================\n$cmd\n";
    $cmd = "$time_exe --output=$file_time --append $cmd"; 
    $fh_err = new IO::File $file_err,'a';
    print $msg;
    print $fh_err $msg;
 	$fh_err -> close();
    system("$cmd");
    
    chdir $sav_dir;
}

=head2 Procedure RunGlintAGSO

 Usage:     &RunGlintAGSO($o_param, $target_dir);
 Function:  Run glint self to compare the bank against itself, auto index mode
 Arg :      $o_param, $target_dir

=cut
sub RunGlintAGSO
{
	my ($o_param,$target_dir) = @_;

	my $sav_dir    = File::Spec->rel2abs(File::Spec->curdir());
	my $data_dir    = File::Spec->rel2abs($o_param->Get('data_directory'));
	my $path_bank   = File::Spec->file_name_is_absolute( $o_param->Get('bank') ) ? 
						$o_param->Get('bank')
						:
						File::Spec->catfile($data_dir,$o_param->Get('bank'));
	my ($b1,$b2,$bank) = File::Spec->splitpath( $path_bank );
	
    my $time_exe       = &TIME_EXE . ' ' . &TIME_OPT;
   	my $glintindex_exe = $o_param -> Get ( 'glintindex_exe' );	
	my $glint_exe      = $o_param -> Get ( 'glint_exe' );

	my $glintindex_opt = $o_param->Get('glintindex_opt');
	my $glint_opt      = $o_param->Get('glint_opt');
	
	chdir $target_dir;
	warn "Now working in directory $target_dir\n";

    # Create a symlink to the bank avoiding a copy
	symlink ( $path_bank, $bank       );

    my $prefix    = &ComputePrefix();
    my $file_time = Sufine::GetFileName(-name => ">${prefix}_time",-ext => 'time');
    my $fh_out    = New Sufine(-name => ">${prefix}_out",-ext => 'out');
    $fh_out -> close();
    my $file_out  = $fh_out->GetFileName();
	$glint_opt   .= " -o $file_out";

    my $fh_err   = New Sufine(-name => ">${prefix}_err",-ext => 'err');
    my $file_err = $fh_err->GetFileName();
    print $fh_err "TESTING glint SELF MODE auto -o switch\n";
    print $fh_err "glint      = $glint_exe $glint_opt\n";
     
    my ($cmd,$msg);
    
    # Executing glint
    $cmd = "$glint_exe $glint_opt $bank 2>>$file_err"; 
    $msg = "STARTING glint\n================\n$cmd\n";
    $cmd = "$time_exe --output=$file_time --append $cmd"; 
    $fh_err = new IO::File $file_err,'a';
    print $msg;
    print $fh_err $msg;
 	$fh_err -> close();
    system("$cmd");
    
    chdir $sav_dir;
}
   
=pod

=head2 Procedure CompareResults

 Usage:     &CompareResults($o_param, $target_dir);
 Function:  compare the results of the last run agains the results found in the reference directory
            3 files are compared/displayed: the output, the stderr, and the execution time
 Arg :      $o_param, $target_dir

=cut
sub CompareResults
{   
	my ($o_param, $target_dir) = @_;
	
    # The reference files
    my $reference_dir	= File::Spec->rel2abs($o_param->Get('reference_directory'));
    my $file_ref_out = File::Spec -> catfile ( $reference_dir, 'output');
    if ( $o_param -> IsDefined ( 'ncpus' ) )
    {
		my $new_file_ref_out = File::Spec -> catfile ( $target_dir, 'output' );
		`cp $file_ref_out $new_file_ref_out `;
		&Sort($new_file_ref_out);
		$file_ref_out = $new_file_ref_out;
	}
	
    my $file_ref_err = File::Spec -> catfile ( $reference_dir, 'error');
    my $file_ref_time= File::Spec -> catfile ( $reference_dir, 'time');
    
    unless ( -r $file_ref_out && -r $file_ref_err && -r $file_ref_time )
    {
        die "Something wrong with the reference files $reference_dir";
    };
    
    chdir $target_dir;
	#warn "Now working in directory $target_dir\n";

    # The target files -> Getting the LAST Sufine version
    my $target_prefix = &ComputePrefix($target_dir);
    my $file_target_out = Sufine::GetPath(-name => "${target_prefix}_out");
    my $file_target_err = Sufine::GetPath(-name => "${target_prefix}_err");
    my $file_target_time= Sufine::GetPath(-name => "${target_prefix}_time");

    if ( $o_param -> IsDefined ( 'ncpus' ) )
    {
		&Sort($file_target_out);
	}
	unless ( -r $file_target_out && -r $file_target_err && -r $file_target_time )
    {
        die "Something wrong with the target files $target_dir";
    };
    
    # Compare them: out and err should be nearly equal, time should be different
    
    my $fh_diff = New Sufine(-name=>">diff", -ext=>"diff");
    my $file_diff = $fh_diff->GetPath();
    
    print $fh_diff "COMPARING RESULTS OF glintindex AND glint\n";
    print $fh_diff "=========================================\n";
    
    print $fh_diff "\nREFERENCE DIRECTORY = $reference_dir\n";
    print $fh_diff "===================\n";
    print $fh_diff "Output              = $file_ref_out\n";
    print $fh_diff "Error               = $file_ref_err\n";
    print $fh_diff "Time                = $file_ref_time\n";
    
    print $fh_diff "\nTARGET DIRECTORY = $target_dir\n";
    print $fh_diff "================\n";
    print $fh_diff "Output              = $file_target_out\n";
    print $fh_diff "Error               = $file_target_err\n";
    print $fh_diff "Time                = $file_target_time\n";
    
    print $fh_diff "\nOUTPUT FILE (REF is < TARGET is >\n";
    print $fh_diff "==================================\n";
    $fh_diff -> close();
    
    system("diff -W200 -y $file_ref_out $file_target_out >> $file_diff");

    $fh_diff =new IO::File(">>$file_diff");    
    print $fh_diff "\nERROR FILE (REF is < TARGET is > )\n";
    print $fh_diff "==================================\n";
    $fh_diff -> close();
    
    system("diff -y $file_ref_err $file_target_err >> $file_diff");
    
    $fh_diff =new IO::File(">>$file_diff");    
    print $fh_diff "\nTIMEFILE, REF:\n";
    &Cat ($fh_diff, $file_ref_time);
    #print $fh_diff `cat $file_ref_time`;
    
    print $fh_diff "\nTIMEFILE, TARGET:\n";
    &Cat($fh_diff, $file_target_time);
    #print $fh_diff `cat $file_target_time`;
    
    $fh_diff-> close();
	if ( $o_param -> IsDefined ( 'only_output' ) )
	{
	    system("diff -I 'glint' -I 'glintindex' $file_ref_out $file_target_out");
	}
	else
	{
	    &Cat(undef,$file_diff);
	}
}

=head2 Procedure Sort

 Usage:     &Sort( $file_name)
 Function:  Sort the file
 Arg :      $file_name 

=cut
sub Sort
{
	my ($file_name) = @_;
	my $sorted_file_name = $file_name.'.sorted' ;
	`sort $file_name > $sorted_file_name`;
	`mv $sorted_file_name $file_name`;
}

sub Usage
{
    print "Calling glintindex+glint \n";
    print "=========================\n";
    print "test.pl --run \n";
    print "        --data_directory      <data_directory>\n";
    print "        --target_directory    <output directory>\n";
    print "        [--check_svn] add a directory tree, its name is the svn revision\n";
    print "        [--bank=...]     (relative to data directory, default=bank.fasta)\n";
    print "        [--auto_index]   using auto index mode: glintindex is not called\n";
    print "        [--o_switch]     using -o switch for glint instead of writing to stdout (ignored if not used with --auto)\n";
    print "        [--request=...]  (relative to data directory, default=request.fasta)\n";
    print "        [--self]         using self mode, ignoring request)\n";
    print "        [--pipe]         Reading the request through stdin\n";
    print "        [--glint_command]=Command for glint (def compare)\n";
    print "        [--glintindex_exe=<executable for glintindex>]\n";
    print "        [--glint_exe     =<executable for glint>]\n\n";    
    print "        [--glintindex_opt=<options for glintindex>]\n";
    print "        [--glint_opt     =<options for glint>]\n\n";
    print "Testing against reference outputs\n";
    print "=================================\n";
    print "test.pl --compare\n";
    print "        --reference_dir Small/ref\n";
    print "        --target_dir    Small/NN\n";
    print "        [--index=index.fasta] (default=bank.fasta)\n";
    print "        [--data=data.fasta] (default=request.fasta)\n\n";
    print "        [--only_output] prints only the outfile diffs when running, however thhe complete diff file is still stored\n";
}
