#!/bin/bash

#set -v 

execute() {
	echo
	echo "Running ===> $1";
	echo
	eval $1;
}	
display() {
	echo
	echo "Displaying ===> $1";
	test=`$1`;
	if [ "x$test" = "x" ]
	then
		tput setaf 2
		tput bold
		echo "test ok"
		tput init
	else
		tput setaf 1
		tput bold
		echo "BAD TEST"
		tput init
	fi
}
display1() {
	tput init
	echo
	echo "Running and Displaying ===> $1";
	test=`eval $1`;
	if [ "$test" = "Test OK" ]
	then
		tput setaf 2
		tput bold
		echo "test ok"
		tput init
	else
		tput setaf 1
		tput bold
		echo "BAD TEST"
		tput init
	fi
	tput init
}
	
makedir_and_copy_from_default() {
	mkdir -p $2
	cp $1/bank.fasta.* $2
}		

DIR=$1
if [ "x$DIR" = "x" ]
then
   echo "Usage: ./test.sh TARGET_DIR [N] [0/1]";
   echo "       TARGET_DIR = The target directory (required)";
   echo "       N          = Number of cpus to use with glint (optional, default 1)";
   echo "       1          = only compare, DO NOT COMPUTE (only useful to check the result of a previous computation)";
   exit 1;
fi

CPUS=$2
if [ "x$CPUS" != "x" ]
then
	CPUS="--ncpus $CPUS"
fi

ONLY_COMPARE=$3
if [ "$ONLY_COMPARE" = "0" ]
then
    ONLY_COMPARE=""
fi

if [ "x$ONLY_COMPARE" = "x" ]
then

	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/default_$DIR "
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/auto_$DIR --auto_index "
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/C0C0_$DIR --glintindex_opt '--no-lc-filtering' --glint_opt '--no-lc-filtering ' "

	makedir_and_copy_from_default tests/small/target/F_$DIR tests/small/target/F_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/F_$DIR --glintindex_opt '--no-lc-filtering' --glint_opt '--no-lc-filtering -f' "
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/M2_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/M2_$DIR --glint_opt '-m 2' --auto_index"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/M3_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/M3_$DIR --glint_opt '-m 3' --auto_index"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/M8_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/M8_$DIR --glint_opt '-m 8' --auto_index"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/M9_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/M9_$DIR --glint_opt '-m 9' --auto_index"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/Msam_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/Msam_$DIR --glint_opt '-m sam' --auto_index --o_switch"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/Mbam_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/Mbam_$DIR --glint_opt '-m bam' --auto_index --o_switch"
	
	execute "./test.pl --run --data_dir tests/self/data  --target tests/self/target/self_$DIR --self --o_switch"

	execute "./test.pl --run --data_dir tests/self/data  --target tests/self/target/self_auto_$DIR --self --auto_index --glint_opt '--no-lc-filtering' --o_switch"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/PIPE_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/PIPE_$DIR --glintindex_opt '--no-lc-filtering' --glint_opt '--no-lc-filtering' --auto_index --pipe"
	
	execute	"./test.pl --run --data_dir tests/small/data --target tests/small/target/p3_$DIR --glintindex_opt '-p 3' "
	
	execute	"./test.pl --run --data_dir tests/small/datagz --target tests/small/target/gz_$DIR --bank bank.fasta.gz --request request.fasta.gz "
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/lc_$DIR
	execute "./test.pl --run --data_dir tests/small/datalc --target tests/small/target/lc_$DIR --auto_index"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/lctoup_$DIR
	execute "./test.pl --run --data_dir tests/small/datalc --target tests/small/target/lctoup_$DIR --glint_opt '--to-upper' --auto_index"
	
	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/best_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/best_$DIR --glint_opt '--best-score -m 8 ' --auto_index"

	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/Mbed_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/Mbed_$DIR --glint_opt '-m bed' --auto_index"

	makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/nogap_$DIR
	execute "./test.pl --run --data_dir tests/small/data --target tests/small/target/nogap_$DIR --glint_opt '--no-gap' --auto_index"
	
	execute "./test.pl --run --data_dir tests/bam/data --target tests/bam/target/bam_$DIR --request=request.bam --glint_opt='--input-format=bam -m 8'"
	
	execute "./test.pl --run --data_dir tests/bam/data --target tests/bam/target/bedpe_$DIR --request=request.bam --glint_command='mappe' --glint_opt='--input-format=bam -m bed ' $CPUS "

	execute "./test.pl --run --data_dir tests/freq/data --target tests/freq/target/default_$DIR --glint_command='freq' --glint_opt='--suitable_min_size 100'"

fi

echo "NOW COMPARING RESULTS - (NO OUTPUT is OK)"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/default_ --target tests/small/target/default_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/auto_ --target tests/small/target/auto_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/C0C0_ --target tests/small/target/C0C0_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/F_    --target tests/small/target/F_$DIR"

display "./test.pl --compare --only_output --reference_dir tests/small/reference/M2_ --target tests/small/target/M2_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/M3_ --target tests/small/target/M3_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/M8_ --target tests/small/target/M8_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/M9_ --target tests/small/target/M9_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/Msam_ --target tests/small/target/Msam_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/Mbam_ --target tests/small/target/Mbam_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/self/reference --target tests/self/target/self_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/self/reference --target tests/self/target/self_auto_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/PIPE_ --target tests/small/target/PIPE_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/p3_ --target tests/small/target/p3_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/gz_ --target tests/small/target/gz_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/lc_ --target tests/small/target/lc_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/lctoup_ --target tests/small/target/lctoup_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/best_ --target tests/small/target/best_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/Mbed_ --target tests/small/target/Mbed_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/small/reference/nogap_ --target tests/small/target/nogap_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/bam/reference/bam_ --target tests/bam/target/bam_$DIR"
display "./test.pl --compare --only_output --reference_dir tests/bam/reference/bedpe_ --target tests/bam/target/bedpe_$DIR $CPUS"
display "./test.pl --compare --only_output --reference_dir tests/freq/reference/default_ --target tests/freq/target/default_$DIR"



# TestConvert.pl
makedir_and_copy_from_default tests/small/target/default_$DIR tests/small/target/convert_$DIR
display1 "./testconvert.pl --run --data tests/small/data --output_format 1 --target tests/small/target/convert_$DIR --glint_opt '--no-lc-filtering $CPUS '"
exit 0

